@@ -4,13 +4,15 @@ from sqlalchemy.engine import Result, Connection
 from sqlalchemy.ext.asyncio import AsyncSession, AsyncEngine, create_async_engine
 from sqlalchemy.future import Engine, create_engine
 from sqlalchemy.orm import sessionmaker, Session
-from sqlalchemy.sql import Executable
+from sqlalchemy.sql import Executable, Select
 from typing_extensions import Concatenate, ParamSpec
 
 from sqlalchemy_database._abc_async_database import AbcAsyncDatabase
 
-_T = TypeVar("_T")
 _P = ParamSpec("_P")
+_T = TypeVar("_T")
+_R = TypeVar("_R")
+
 _ExecuteParams = Union[Mapping[Any, Any], Sequence[Mapping[Any, Any]]]
 _ExecuteOptions = Mapping[Any, Any]
 
@@ -38,24 +40,32 @@ class AsyncDatabase(AbcAsyncDatabase):
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             bind_arguments: Optional[Mapping[str, Any]] = None,
-            commit: bool = False,
+            commit: bool = True,
             on_close_pre: Callable[[Result], _T] = None,
+            is_session: bool = True,
             **kw: Any,
     ) -> Union[Result, _T]:
-        async with self.session_maker() as session:
-            result = await session.execute(statement, params, execution_options, bind_arguments, **kw)
-            if commit:
-                await session.commit()
+        if is_session:
+            maker = self.session_maker
+            kw['bind_arguments'] = bind_arguments
+        else:
+            maker = self.engine.connect
+        async with maker() as conn:
+            result = await conn.execute(statement, params, execution_options, **kw)  # type:ignore
             if on_close_pre:
                 result = on_close_pre(result)
+            if commit and not isinstance(statement, Select):
+                await conn.commit()
         return result
 
     async def scalar(
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             bind_arguments: Optional[Mapping[str, Any]] = None,
             **kw: Any,
@@ -73,6 +83,7 @@ class AsyncDatabase(AbcAsyncDatabase):
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             **kw: Any,
     ) -> List[Any]:
@@ -89,6 +100,7 @@ class AsyncDatabase(AbcAsyncDatabase):
             self,
             entity: Type[_T],
             ident: Any,
+            *,
             options: Optional[Sequence[Any]] = None,
             populate_existing: bool = False,
             with_for_update: Optional[Any] = None,
@@ -110,19 +122,28 @@ class AsyncDatabase(AbcAsyncDatabase):
             async with session.begin():
                 await session.delete(instance)
 
+    async def save(self, *instances: Any) -> None:
+        async with self.session_maker() as session:
+            async with session.begin():
+                session.add_all(instances)
+
     async def run_sync(
             self,
             fn: Callable[[Concatenate[Union[Session, Connection], _P]], _T],
             *args: _P.args,
-            is_session: bool = False,
+            commit: bool = True,
+            on_close_pre: Callable[[_T], _R] = None,
+            is_session: bool = True,
             **kwargs: _P.kwargs
-    ) -> _T:
-        if is_session:
-            async with self.session_maker() as session:
-                async with session.begin():
-                    return await session.run_sync(fn, *args, **kwargs)
-        async with self.engine.begin() as conn:
-            return await conn.run_sync(fn, *args, **kwargs)
+    ) -> Union[_T, _R]:
+        maker = self.session_maker if is_session else self.engine.connect
+        async with maker() as conn:
+            result = await conn.run_sync(fn, *args, **kwargs)
+            if on_close_pre:
+                result = on_close_pre(result)
+            if commit:
+                await conn.commit()
+        return result
 
 
 class Database(AbcAsyncDatabase):
@@ -147,24 +168,32 @@ class Database(AbcAsyncDatabase):
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             bind_arguments: Optional[Mapping[str, Any]] = None,
-            commit: bool = False,
+            commit: bool = True,
             on_close_pre: Callable[[Result], _T] = None,
+            is_session: bool = True,
             **kw: Any,
     ) -> Union[Result, _T]:
-        with self.session_maker() as session:
-            result = session.execute(statement, params, execution_options, bind_arguments, **kw)
-            if commit:
-                session.commit()
+        if is_session:
+            maker = self.session_maker
+            kw['bind_arguments'] = bind_arguments
+        else:
+            maker = self.engine.connect
+        with maker() as conn:
+            result = conn.execute(statement, params, execution_options, **kw)
             if on_close_pre:
                 result = on_close_pre(result)
+            if commit and not isinstance(statement, Select):
+                conn.commit()
         return result
 
     def scalar(
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             bind_arguments: Optional[Mapping[str, Any]] = None,
             **kw: Any,
@@ -182,6 +211,7 @@ class Database(AbcAsyncDatabase):
             self,
             statement: Executable,
             params: Optional[_ExecuteParams] = None,
+            *,
             execution_options: Optional[_ExecuteOptions] = None,
             bind_arguments: Optional[Mapping[str, Any]] = None,
             **kw: Any,
@@ -199,6 +229,7 @@ class Database(AbcAsyncDatabase):
             self,
             entity: Type[_T],
             ident: Any,
+            *,
             options: Optional[Sequence[Any]] = None,
             populate_existing: bool = False,
             with_for_update: Optional[Any] = None,
@@ -220,16 +251,25 @@ class Database(AbcAsyncDatabase):
             with session.begin():
                 session.delete(instance)
 
+    def save(self, *instances: Any) -> None:
+        with self.session_maker() as session:
+            with session.begin():
+                session.add_all(instances)
+
     def run_sync(
             self,
             fn: Callable[[Concatenate[Union[Session, Connection], _P]], _T],
             *args: _P.args,
-            is_session: bool = False,
+            commit: bool = True,
+            on_close_pre: Callable[[_T], _R] = None,
+            is_session: bool = True,
             **kwargs: _P.kwargs
-    ) -> _T:
-        if is_session:
-            with self.session_maker() as session:
-                with session.begin():
-                    return fn(session, *args, **kwargs)
-        with self.engine.begin() as conn:
-            return fn(conn, *args, **kwargs)
+    ) -> Union[_T, _R]:
+        maker = self.session_maker if is_session else self.engine.connect
+        with maker() as conn:
+            result = fn(conn, *args, **kwargs)
+            if on_close_pre:
+                result = on_close_pre(result)
+            if commit:
+                conn.commit()
+        return result
